﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class FireInvoker : MonoBehaviour
{
	public float checkRadius = 0.3f;
	public float fireDelay = 0.2f;
	public float prefabLife = 1.0f;
	public float prefabInterval = 0.1f;
	public int prefabCount = 20;
	public GameObject firePrefab;
	public AudioClip soundKaboom;
	public AudioClip soundNoTrains;

	private Transform _transform;
	private bool isFiring = false;

	public void Awake()
	{
		_transform = transform;
	}

	public IEnumerator DoFire()
	{
		if (isFiring)
			yield break;

		if (GameLogic.instance.boltCount <= 0)
			yield break;
		GameLogic.instance.UpdateBoltCount(-1);

		isFiring = true;
		yield return new WaitForSeconds(fireDelay);

		var pos = _transform.position;
		var colliders = Physics2D.OverlapCircleAll(pos, checkRadius, 1<<10);
		if (colliders.Length != 0)
		{
			AudioSource.PlayClipAtPoint(soundKaboom, Camera.main.transform.position);
		}
		else
		{
			AudioSource.PlayClipAtPoint(soundNoTrains, Camera.main.transform.position);
		}
		foreach (var c in colliders)
		{
			StartCoroutine(DestroyObject(c));
		}
		isFiring = false;
	}

	private IEnumerator DestroyObject(Collider2D col)
	{
		col.gameObject.layer = 0; // stop future fires from finding us while we're in process of destruction :)
		var sprite = col.GetComponent<SpriteRenderer>();
		var bounds = col.bounds;
		var spawned = 0;
		GameObject go = null;
		for (var i = 0; i < prefabCount*5 && spawned < prefabCount; ++i)
		{
			var point = new Vector2(Random.Range(bounds.min.x, bounds.max.x), Random.Range(bounds.min.y, bounds.max.y));
			if (col.OverlapPoint(point))
			{
				go = Instantiate(firePrefab, point, Quaternion.Euler(0,0,Random.Range(0f,360f)));
				Destroy(go, prefabLife);
				go.transform.position = point;
				sprite.color = new Color(1,1,1,1.0f-(float)spawned/(float)prefabCount);
				yield return new WaitForSeconds(prefabInterval);
				++spawned;
			}
		}
		Destroy(col.gameObject);
		yield return 0;
		GameLogic.instance.UpdateTrainCount();
	}
}
